using System;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Http;

namespace DevelopMentor {

    // IDigitSeries
    //
    // Iinterface representing a series of digits between a
    // starting and ending value, based on some algorithm.
    //
    public interface IDigitSeries
    {
        int first {
            get;
        }

        int last {
            get;
        }

        void Output( System.IO.Stream s );
    }

    // IDigitSeriesCreator
    //
    // Interface that allows a digit series to be created
    // and returned to the caller.
    //
    public interface IDigitSeriesCreator
    {
        IDigitSeries Create( int first, int last );
    }
    
    // IntegerSeries
    //
    // A class that implements a simple integer counting between
    // two values.  Instances of this type will be marshalled by
    // value.  The runtime will marshal the nFirst and nLast fields
    // of this type for us, but will ignore theSeries, since it
    // is marked [NonSerialized].
    //
    [Serializable]
    class IntegerSeries : IDigitSeries,
                          IDeserializationCallback
    {
        public int first {
            get { return nFirst; }
        }
        
        public int last {
            get { return nLast; }
        }

        public void Output( System.IO.Stream s )
        {
            StreamWriter writer = new StreamWriter(s);

            writer.WriteLine("Series from {0} to {0}:", nFirst, nLast);

            if( theSeries != null )
            {
                foreach( int n in theSeries )
                {
                    writer.WriteLine(n);
                }
            }
            else
            {
                writer.WriteLine("Series empty.");
            }

            writer.Flush();
        }

        // The constructor will be invoked locally on the server when
        // SeriesFactory.Create is called.  This constructor will not
        // be invoked in the target context.
        //
        public IntegerSeries( int first, int last )
        {
            Console.WriteLine("IntegerSeries constructor");

            nFirst = first;
            nLast = last;

            GenerateSeries();
        }

        void GenerateSeries()
        {
            int count = nLast - nFirst + 1;

            theSeries = new int[count];

            for( int n = 0; n < count; n++ )
            {
                theSeries[n] = nFirst + n;
            }
        }

        // OnDeserialization will be called in the target context
        // by the runtime after runtime-managed deserialization of
        // this type is complete, but before an object reference
        // is returned to the caller.  We'll take this opportunity
        // to generate the series locally in the client's context.
        //
        public virtual void OnDeserialization( object sender )
        {
            Console.WriteLine("IntegerSeries.OnDeserialization");
            GenerateSeries();
        }

        // The following member variables will be marshalled by 
        // the runtime.
        //
        private int nFirst;
        private int nLast;

        // The runtime will skip this member during serialization.
        // We'll fix it up in OnDeserialization.
        //
        [NonSerialized] private int[] theSeries;
    }

    // SeriesFactory
    //
    // Instances of this type are marshalled by reference back to the client
    // and used to create series objects, which are marshalled by value.
    //
    public class SeriesFactory : MarshalByRefObject, IDigitSeriesCreator
    {
        public IDigitSeries Create( int first, int last )
        {
            return new IntegerSeries(first, last);
        }
    }

    public class MyServer
    {
        public static int Main(string [] args)
        {
            HttpChannel chan = new HttpChannel(999);

            ChannelServices.RegisterChannel(chan);
            RemotingConfiguration.RegisterWellKnownServiceType(
                typeof(SeriesFactory),
                "myendpoint", WellKnownObjectMode.Singleton );

            // Block the primary thread until the ENTER key is pressed.
            Console.WriteLine("Hit <enter> to exit...");
            Console.ReadLine();
            return 0;
        }
    }
}

